#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

# Usage: perl gen-curve.pl 
#        perl gen-curve.pl p
#
# Print PGF/Tikz drawing for Mblock(p) interpreted as segment directions in
# the alternate paperfolding curve.

use 5.010;
use strict;
use warnings;
use File::Spec;

use FindBin;
use lib $FindBin::Bin;
use MyAppendMorphism 'Mblock','@s_to_dx','@s_to_dy';
use MyPrintwrap ':all';
$|=1;


my $p = 5;
if (@ARGV) { $p = shift @ARGV; }
$p >= 1 or die "must have p >= 1";

print "  % generated by tools/$FindBin::Script $p\n";
print "  \\newcommand\\MyP{$p}\n";

sub chamfer {
  my ($delta, $name) = @_;
  $name //= '\MyChamfer';
  if ($delta == 0) { return ''; }
  if ($delta == 1) { return "+$name"; }
  if ($delta == -1) { return "-$name"; }
  return sprintf '%+d*%s', $delta, $name;
}

my @Mblock = Mblock($p);
my (@x,@y,@dx,@dy);
{
  my %seen;
  my $x = 0;
  my $y = 0;
  foreach my $i (0 .. $#Mblock) {
    my $s = $Mblock[$i];
    $x[$i] = $x;
    $y[$i] = $y;
    my $dx = $dx[$i] = $s_to_dx[$s];
    my $dy = $dy[$i] = $s_to_dy[$s];
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    $seen{"$x,$y,$x2,$y2"}++;
    $seen{"$x2,$y2,$x,$y"}++;
    ($x,$y) = ($x2,$y2);
  }
  my $bad = 0;
  while (my ($key,$reps) = each %seen) {
    if ($reps > 2) {
      print "bad $key has $reps repeat traversals\n";
      $bad++;
    }
  }
  $bad==0 or die;
}
# $singles->remove_segments($doubles);

my $len = @Mblock;
my $eighth = $len >> 3;


#     7 <----- 6 2 ----> 3
#     |        ^ ^       |         double traversed
#     |        | |       |         1to2 and 5to6
#     v        | |       v
#     0 -----> 1 5 <---- 4

foreach my $elem ([ 0, 1, 'my single'],
                  [ 1, 2, 'my double'],
                  [ 2, 5, 'my single'],
                  [ 6, 8, 'my single']) {
  my ($start,$end,$style) = @$elem;
  my $start_i = $eighth * $start;
  my $end_i   = $eighth * $end;

  $Printwrap = 2;
  printwrap("\\draw[$style] ");
  $Printwrap += 2;
  my $join = '';
  if ($style eq 'my single' && $start != 0) {
    my $px = $x[$start_i] . chamfer(-$dx[$start_i-1]);
    my $py = $y[$start_i] . chamfer(-$dy[$start_i-1]);
    printwrap("($px,$py)");
    $join = '--';
  }
  foreach my $i ($start_i .. $end_i-1) {
    my $sx = $x[$i] . chamfer($dx[$i]);
    my $sy = $y[$i] . chamfer($dy[$i]);
    my $tx = ($x[$i]+$dx[$i]) . chamfer(-$dx[$i]);
    my $ty = ($y[$i]+$dy[$i]) . chamfer(-$dy[$i]);
    printwrap("$join($sx,$sy)--($tx,$ty)");
    $join = '--';
  }
  if ($style eq 'my single' && $end != 8) {
    my $tx = $x[$end_i] . chamfer($dx[$end_i]);
    my $ty = $y[$end_i] . chamfer($dy[$end_i]);
    printwrap("$join($tx,$ty)");
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
